import re

"""
@features extracts the feature set for a single token in the document.
params:
	entry : single token from the current document, string
	index : position of the token in the document, ranging from 0 to len(document), integer
	document: complete text of the current seminar announcement, list of strings
returns:
	feature_dict : dictionary of features for the current token 'entry'

This function extracts features for a single token and saves them in the form of a feature dictionary
which will later get used by the learner to classify our training, validation and test sets.

Your task is to find good features for our perceptron learner to classify the tokens as either part
of a location or not. Every single token needs a dictionary of features and the features you extract
for each token are the same. Please be aware that the system will only "see' whatever information
you provide through the features.

If you want to import other libraries to help you with your features you can do so at the top of the file.
Python regex is already imported for your convenience, but you do not need to use it if you don't want to.
"""

def features(entry, index, document):
	feature_dict = {}

	######### YOUR CODE HERE #########
	# These are two example features to show you how you can add a feature for a single token.
	# Hint: These are not good features and your system might not need them at all.
	feature_dict['isRoom'] = entry == 'Room'
	feature_dict['isUpper'] = entry.isupper()
	
	
	######### END OF YOUR CODE #########

	return feature_dict

"""
@feature_extractor processes the entire corpus for feature extraction
params:
	fullset : list of documents
returns:
	raw_features : list of dictionaries, one dictionary for each token in each document

A document is a list of tokens in the order it was written in the announcements.
The function output is expected to be a list of dictionaries, which we need for the sklearn vectorizer. (raw_features)
Each entry in each document will get its own vector of features, represented by the dictionary.
The actual features are extracted in @features

!!! NOTHING TO DO HERE !!!
"""

def feature_extractor(fullset):
	raw_features = list()
	for document in fullset:
		for i, entry in enumerate(document):
			raw_features.append(features(entry, i, document))
	return raw_features
