import sys
import numpy as np
from feature_extractor import *
from pystruct.learners import StructuredPerceptron
from pystruct.models import MultiClassClf
from sklearn.feature_extraction import DictVectorizer

def readfile(filename):
	filecontents = list()
	with open(filename) as fh:
		doccontent = list()
		for line in fh:
			if len(line) == 1:
				filecontents.append(doccontent)
				doccontent = list()
			else:
				splitline = line.split()
				if len(splitline) > 1:
					doccontent.append((splitline[0], splitline[1]))
	return [x for x in filecontents if x != []]

def extract_labels(fullset):
	rawlabels = list()
	for document in fullset:
		for entry in document:
			rawlabels.append(entry[1])
	return rawlabels

def get_documents(fullset):
	docs = list()
	for document in fullset:
		entries = list()
		for entry in document:
			entries.append(entry[0])
		docs.append(entries)
	return docs

def flat_words(fullset):
	allwords = list()
	for document in fullset:
		for entry in document:
			allwords.append(entry[0])
	return allwords

def switch_labels(labels):
	newlabels = list()
	for label in labels:
		if label == 'O':
			newlabels.append(0)
		elif label == 'B':
			newlabels.append(1)
		elif label == 'I':
			newlabels.append(2)
	return newlabels

def train_perceptron(features, labels):
	label_array = np.asarray(switch_labels(labels)).reshape(-1,1)
	print(features.shape)
	print(features)
	print(label_array.shape)
	learner = StructuredPerceptron(MultiClassClf())
	learner.fit(features, label_array)
	return learner

def scorer(goldlabels, predictedlabels):
	truepos = 0
	falsepos = 0
	falseneg = 0
	for i, label in enumerate(predictedlabels):
		if label == 'B' and goldlabels[i] == 'B' or label == 'I' and goldlabels[i] == 'I':
			truepos += 1
		elif label == 'B' or label == 'I':
			falsepos += 1
		elif goldlabels[i] == 'B' or goldlabels[i] == 'I':
			falseneg += 1
	try:
		precision = truepos / (truepos + falsepos)
		recall = truepos / (truepos + falseneg)
	except ZeroDivisionError:
		precision = 0.00001
		recall = 0.00001

	try:
		f1 = 2 * (precision * recall) / (precision + recall)
	except ZeroDivisionError:
		f1 = 0

	print('true positives:', truepos)
	print('false positives:', falsepos)
	print('false negatives:', falseneg)

	print('Precision:', precision)
	print('Recall:', recall)
	print('F1:', f1)


def writepredictions(flatwords, predictedlabels, filename):
	with open(filename, 'w') as fh:
		for i, word in enumerate(flatwords):
			fh.write(word + "\t" + predictedlabels[i] + '\n')

def main():
	vectorizer = DictVectorizer(sparse = True)
	trainset = readfile(sys.argv[1])
	devset = readfile(sys.argv[2])
	testset = readfile(sys.argv[3])

	labels = extract_labels(trainset)

	trainwords = get_documents(trainset)
	devwords = get_documents(devset)
	testwords = get_documents(testset)

	raw_features = feature_extractor(trainwords)
	features = vectorizer.fit_transform(raw_features)
	perceptron = train_perceptron(features, labels)

	raw_features_predict = feature_extractor(devwords)
	features_predict = vectorizer.transform(raw_features_predict)
	predictions = perceptron.predict(features_predict)

	"""
	raw_features_test = feature_extractor(testwords)
	test_predict = vectorizer.transform(raw_features_test)
	predictions = perceptron.predict(test_predict)
	"""

	words = flat_words(devset) # enter testset for final check
	labels = extract_labels(devset) # enter testset

	scorer(labels, predictions)

	writepredictions(words, labels, 'standard.txt')
	writepredictions(words, predictions, 'predictions.txt')


if __name__ == '__main__':
	main()